/*
 * Copyright (C) 2000 Erik Cumps
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 * $Id: ipquarn.c,v 1.7 2000/08/14 19:40:46 slay Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <signal.h>
#include <getopt.h>
#include <pwd.h>
#include <sys/types.h>
#include "ipquota.h"
#include "pathnames.h"
#include <midbs.h>

/* put here to satisfy hash.c */
int skfd = -1;

/* --- global variables --- */
int   verbose = 0;
char  defaultdb[2048] = _PATH_VAR_LOG_QUOTA_STATS;
char *database;

void usage(void)
{
    fprintf(stderr,"Usage: %s [-vhV]\n", pname);
    exit(EXIT_FAILURE);
}

void catch_sig(int sig)
{
    close_database(database);
    mylog("terminating (signal %i)", sig);
    exit(EXIT_SUCCESS);
}

void check_user (struct user_info *ui)
{
    struct passwd *pwd = NULL;

    if (ui->warn & WARN) {
	if (verbose) {
	    pwd = getpwuid(ui->uid);
	    if (pwd != NULL) {
		printf ("\t%s\n", pwd->pw_name);
	    } else {
		printf ("unknown:%u\n", ui->uid);
	    }
	}
	mylog ("warning triggered for user %u -> modify check_user() function in ipquota/ipquarn.c!", ui->uid);
	/* now that you have the uid and username of a user 
	   for whom a warning was triggered, you could do
	   something useful here, like, e.g.:

	   char buffer[128];
	   sprintf (buffer, "mail -s \"ipquota soft quota warning\" %s", pwd->pw_name);
	   system (buffer); 

	*/
    }
    return;
}

void check_all_users (void)
{
    search_handle sh;
    struct search_info si;
    struct user_info ui;
    int rc;

    if (MIDBS_allocsearch (&sh) == -1) {
	mylog ("check_all_users: error: no backend module linked in!");
	exit (EXIT_FAILURE);
    }

    si.uid_first = 0;
    si.uid_last  = 1;
    rc = MIDBS_findfirst (sh, &si, &ui);

    if (rc == -2) {
	MIDBS_freesearch (&sh);
	return;
    }

    check_user (&ui);
    while (MIDBS_findnext (sh, &ui) != -2) {
	check_user (&ui);
    }
    MIDBS_freesearch (&sh);

    return;
}

int main (int argc,char **argv) 
{
    extern int optind; 
    extern char *optarg;
    int c;

    database = getenv ("IPQUOTADB");
    if (database == NULL) { database = defaultdb; }
	
    use_syslog = 0;	
    pname = argv[0];
    openlog(argv[0], LOG_PID, LOG_DAEMON);

    while ((c = getopt(argc,argv,"vhV")) > 0) 
  	{
	    switch (c)
  		{
		case 'V':
		    verbose++;
		    break;
		case 'v':
		    version();
		    break; /* never reached */
		case 'h':
		default:
		    usage();
		    break; /* never reached */
  		}
  	}

    if (verbose) {
	printf ("Warnings triggered for:\n");
    }

    signal(SIGINT, catch_sig);
    signal(SIGTERM, SIG_IGN); /* ignore this so that we can
				 finish our last accounting run */
    signal(SIGQUIT, catch_sig);
    signal(SIGHUP, catch_sig);

    init_hash();
    init_database(database);	

    check_all_users ();
	
    close_database(database);
    closelog();

    return EXIT_SUCCESS;
}

