/*
 * Modular IP quota Database Backend System
 * Copyright (C) 2000 Erik Cumps
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 * MIDBS header file (private interface to backend modules)
 *
 * $Id: midbs_private.h,v 1.11 2000/09/01 15:27:09 slay Exp $
 */

#ifndef __MIDBS_PRIVATE_H
  #define __MIDBS_PRIVATE_H

/* Version Control */
#ifndef lint
static const char midbs_private_h_vcid[] = "$Id: midbs_private.h,v 1.11 2000/09/01 15:27:09 slay Exp $";
#endif /* lint */

#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <midbs.h>

#ifdef BENCHMARK
#include <sys/times.h>
#endif /* BENCHMARK */

/* --- Tuning parameters --- */
#define LOCK_TRIES   20   /* maximum tries to lock database */

/* --- Benchmark structures --- */
#ifdef BENCHMARK
struct MIDBS_BM {
    clock_t    clk_start, clk_stop;
    struct tms tms_start, tms_stop;
};
extern unsigned long MIDBS_db_entries;
extern unsigned long MIDBS_db_queries;
#endif /* BENCHMARK */

/* --- Exported private API --- */
int  MIDBS_private_open        (char *);
int  MIDBS_private_close       (char *);
int  MIDBS_private_allocsearch (search_handle *);
int  MIDBS_private_freesearch  (search_handle *);
int  MIDBS_private_findfirst   (search_handle, struct search_info *, struct user_info *);
int  MIDBS_private_findnext    (search_handle, struct user_info *);
int  MIDBS_private_write       (struct user_info *);
int  MIDBS_private_delete      (struct user_info *);

/* --- API stub functions --- */
int  MIDBS_open_stub           (char *);
int  MIDBS_close_stub          (char *);
int  MIDBS_allocsearch_stub    (search_handle *);
int  MIDBS_freesearch_stub     (search_handle *);
int  MIDBS_findfirst_stub      (search_handle, struct search_info *, struct user_info *);
int  MIDBS_findnext_stub       (search_handle, struct user_info *);
int  MIDBS_write_stub          (struct user_info *);
int  MIDBS_delete_stub         (struct user_info *);

/* --- MIDBS private functions --- */
int  MIDBS_ctrl_access         (uid_t, int);
void CheckAndDropSuid          (void);
int  FetchReqConnectionStr     (char *, char *, char **);
int  FetchConnectionVal        (char *, char *, long *);

#ifdef BENCHMARK
void MIDBS_benchmark_report (struct MIDBS_BM *, char *, unsigned long, char *);
#endif /* BENCHMARK */

#endif /* __MIDBS_PRIVATE_H */

