/*
 * Modular IP quota Database Backend System
 * Copyright (C) 2000 Erik Cumps
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 * MIDBS MySQL header file
 *
 * $Id: midbs_mysql.h,v 1.2 2000/08/14 19:41:00 slay Exp $
 */

#ifndef __MIDBS_MYSQL_H
#define __MIDBS_MYSQL_H

/* Version Control */
#ifndef lint
static const char midbs_mysql_h_vcid[] = "$Id: midbs_mysql.h,v 1.2 2000/08/14 19:41:00 slay Exp $";
#endif /* lint */

/* MySQL handling macros */

/* Reports an SQL error
   Input: func: string with name of calling function
          msg : string with own error message
*/
#define REPORT_SQL_ERROR(func, msg) \
    mylog ("%s: %s: error: %s (%u)", func, msg, mysql_error (&mysql), mysql_errno (&mysql));

/* Checks for and reports an SQL error, with return
   Input: func  : string with name of calling function
          msg   : string with own error message
          object: pointer to mysql object
          rc    : value to return with
 */
#define CHECK_SQL_ERROR_R(func, msg, object, rc) \
    if (mysql_errno(object)) {                 \
        REPORT_SQL_ERROR(func, msg);           \
	return rc;                             \
    }

/* Checks for and reports an SQL error, *without* return
   Input: func  : string with name of calling function
          msg   : string with own error message
          object: pointer to mysql object
          rc    : variable to store result of check in
 */
#define CHECK_SQL_ERROR(func, msg, object, rc) \
    if (mysql_errno(object)) {                 \
        REPORT_SQL_ERROR(func, msg);           \
        rc = 1;                                \
    } else {                                   \
        rc = 0;                                \
    }

/* Reports an SQL query error
   Input: func : string with name of calling function
          query: string with current SQL query
 */
#define REPORT_QUERY_ERROR(func, query)	                                                  \
    mylog ("%s: query error: %s (%u)", func, mysql_error (&mysql), mysql_errno (&mysql)); \
    mylog ("%s: failed query: \"%s\"", func, query);

/* Reports an SQL query error (with msg)
   Input: func : string with name of calling function
          query: string with current SQL query
          msg  : string with own message
 */
#define REPORT_QUERY_ERROR_MSG(func, query, msg)	                                                  \
    mylog ("%s: query error: %s: %s (%u)", func, msg, mysql_error (&mysql), mysql_errno (&mysql)); \
    mylog ("%s: failed query: \"%s\"", func, query);

/* Checks for and reports an SQL query error, with return
   Input: func  : string with name of calling function
          query : string with current query
          object: pointer to mysql object
          rc    : value to return with
 */
#define CHECK_QUERY_ERROR_R(func, query, object, rc) \
    if (mysql_errno(object)) {                     \
        REPORT_QUERY_ERROR(func, query);           \
	return rc;                                 \
    }

/* Checks for and reports an SQL query error, *without* return
   Input: func  : string with name of calling function
          query : string with current query
          object: pointer to mysql object
          rc    : variable to store result of check in
 */
#define CHECK_QUERY_ERROR(func, query, object, rc) \
    if (mysql_errno(object)) {                     \
        REPORT_QUERY_ERROR(func, query);           \
        rc = 1;                                    \
    } else {                                       \
        rc = 0;                                    \
    }

#endif /* __MIDBS_MYSQL_H */

