/*
 * Copyright (C) 1995 Joao Ventura, Lars Fenneberg
 * Portions Copyright (C) 2000 Erik Cumps
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 * $Id: sayip.c,v 1.14 2000/08/14 19:40:46 slay Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <signal.h>
#include <time.h>
#include <pwd.h>
#include <getopt.h>
#include <string.h>
#include <linux/ip_acct_user.h>
#include "ipquota.h"
#include "pathnames.h"

/* Define ALTERNATIVE for alternative -H reporting */
#define ALTERNATIVE	1

/* --- Global variables --- */
struct passwd *pwd = NULL;
int human = 0;
char  defaultdb[2048] = _PATH_VAR_LOG_QUOTA_STATS;
char *database;

struct TTconvert {
    int divisor;
    int modulo;
    char *name;
};

void usage(void)
{
    fprintf(stderr,"Usage: %s [-vhacH] [<username/uid>]\n", pname);
    exit (EXIT_FAILURE);	
}

void catch_sig(int sig)
{
    close_database(database);
    mylog("terminating (signal %i)", sig);
    exit(EXIT_SUCCESS);
}

char *TT_to_human (time_t in, char *s, int full)
{
    static struct TTconvert TU[5] = { { 60, 0, "second" }, 
				      { 60, 0, "minute" }, 
				      { 24, 0, "hour"}, 
				      { 7, 0, "day" }, 
				      { 0, 0, "week" } };
    int i   = 0;
    int len = 0;
    signed long t;
    char buffer[256];

    if (human && (in != 0)) {
	*s = 0;
	t  = in;
	for (i=0; i<4; i++) {
	    TU[i].modulo = t % TU[i].divisor;
	    t = t / TU[i].divisor;
	}
	TU[4].modulo = t;
	t = full + 1;
	for (i=4; i>=0; i--) {
	    if (t && (len || TU[i].modulo)) {
		sprintf (buffer, "%u %s%s, ", TU[i].modulo, TU[i].name, (TU[i].modulo == 1) ? "" : "s");
		strcat (s, buffer);
		if (!len) { t--; }
		len = 1;
	    }
	}
	len = strlen (s) - 2;
	*(s + len) = 0;
    } else {
	len = sprintf (s, "%lu second%s", in, (in == 1) ? "" : "s");
    }
    return (s + len + 1);
}

char *SLL_to_human (signed long long in, char *s, int full)
{
    static struct TTconvert TU[6] = { { 1024, 0, "byte" }, 
				      { 1024, 0, "kilobyte" },
				      { 1024, 0, "megabyte"},
				      { 1024, 0, "gigabyte" },
				      { 1024, 0, "terabyte" },
				      { 0, 0, "petabyte" } };
    int i   = 0;
    int len = 0;
    int sign = 1;
    signed long long t;
    char buffer[256];

    if (human && (in != 0)) {
	if (in < 0) { sign = -1; }
	*s = 0;
	t  = sign * in;
	for (i=0; i<5; i++) {
	    TU[i].modulo = t % TU[i].divisor;
	    t = t / TU[i].divisor;
	}
	TU[5].modulo = t;
	t = full + 1;
	if (sign == -1) {
	    sprintf (buffer, "-");
	    strcat (s, buffer);
	}
	for (i=5; i>=0; i--) {
	    if (t && (len || TU[i].modulo)) {
		sprintf (buffer, "%u %s%s, ", TU[i].modulo, TU[i].name, (TU[i].modulo == 1) ? "" : "s");
		strcat (s, buffer);
		if (!len) { t--; }
		len = 1;
	    }
	}
	len = strlen (s) - 2;
	*(s + len) = 0;
    } else {
	len = sprintf (s, "%Li byte%s", in, (in == 1) ? "" : "s");
    }
    return (s + len + 1);
}

/* Parses option argument and returns struct passwd */
int arg_to_uid (char *arg, struct passwd **result)
{
    uid_t uid;
    char *c;
    int nan = 0;

    if (*arg == 0) {
	mylog ("arg_to_uid(): empty argument");
	exit (EXIT_FAILURE);
    }

    uid = (uid_t) strtol (arg, &c, 10);
    nan = (*c != 0);

    uid = (uid_t) atoi(arg);

    /* deal with "everyone" special case */
    if (strcmp(arg, "everyone") == 0) {
	return 1;
    }

    /* is it a username? */
    *result = getpwnam (arg);
    if (*result != NULL) {
	uid = (*result)->pw_uid;
    } else {
	/* is it a uid? */
	*result = getpwuid (uid);
	if (nan || (*result == NULL)) {
	    /* user not found! */
	    mylog ("no such user/uid: %s", arg);
	    exit (EXIT_FAILURE);
	}
    }

    return 0;
}

void mini_report (struct user_info *ui, int check)
{
    struct passwd *pwd = NULL;

    if ((pwd = getpwuid (ui->uid)) == NULL) {
	if (check) {
	    printf("Invalid user uid=");
	}
	printf("%-6i %Li %Li %lu %lu %i %i\n",
               ui->uid, ui->Hquota, ui->Squota,
               ui->maxT, ui->T0, ui->flags, ui->warn);
    } else {
	if (!check) {
	    printf("%-8s %Li %Li %lu %lu %i %i\n",
                   pwd->pw_name, ui->Hquota, ui->Squota,
                   ui->maxT, ui->T0, ui->flags, ui->warn);
	}
    }
}

void report (struct user_info *ui)
{
    char buffer[1024];
    char *Hq, *Sq, *ma;

    Hq = buffer;
#ifdef ALTERNATIVE
    Sq = SLL_to_human (ui->Hquota, Hq, 1);
    ma = SLL_to_human (ui->Squota, Sq, 1);
         TT_to_human  (ui->maxT,   ma, 1);
#else
    Sq = SLL_to_human (ui->Hquota, Hq, 0);
    ma = SLL_to_human (ui->Squota, Sq, 0);
         TT_to_human  (ui->maxT,   ma, 0);
#endif /* ALTERNATIVE */ 
    printf ("%-8s - ip hard quota: %s left\n",      pwd->pw_name, Hq);
    printf ("         - soft quota   : %s delta\n", Sq);
    printf ("         - grace period : %s\n",       ma);
    if (ui->T0) {
	printf ("         - soft quota reached at %s", ctime(&ui->T0));
    } else {
	if (ui->Hquota <= 0) {
	    printf ("         - hard quota reached before soft quota could be registered\n");
	} else {
	    printf ("         - soft quota not yet reached\n");
	}
    }
    printf ("         - access: %s, warning: %s\n", 
            (ui->flags & IPAU_NO_ACCESS) ? "denied" : "allowed",
            (ui->warn & WARN) ? "sent" : "none");
}

void do_report_all (int check)
{
    search_handle sh;
    struct search_info si;
    struct user_info ui;
    int rc;

    if (MIDBS_allocsearch (&sh) == -1) {
	mylog ("do_report_all: error: no backend module linked in!");
	exit (EXIT_FAILURE);
    }

    si.uid_first = 0;
    si.uid_last  = 1;
    rc = MIDBS_findfirst (sh, &si, &ui);

    if (rc == -2) {
	MIDBS_freesearch (&sh);
	return;
    }

    mini_report (&ui, check);

    while (MIDBS_findnext (sh, &ui) != -2) {
	mini_report (&ui, check);
    }
    MIDBS_freesearch (&sh);

    return;
}

void do_report_user (uid_t uid)
{
    search_handle sh;
    struct search_info si;
    struct user_info ui;
    int rc;

    if (MIDBS_allocsearch (&sh) == -1) {
	mylog ("do_report_user: error: no backend module linked in!");
	exit (EXIT_FAILURE);
    }

    si.uid_first = uid;
    si.uid_last  = uid;
    rc = MIDBS_findfirst (sh, &si, &ui);
    MIDBS_freesearch (&sh);

    if (rc == -2) {
	printf("%s - no ip quota defined.\n", pwd->pw_name);
	return;
    }

    report (&ui);

    return;
}

void do_report (int all, int check)
{
    if (all) {
	do_report_all (check);
    } else {
	do_report_user (pwd->pw_uid);
    }
    return;
}

int main (int argc,char **argv)
{
    int c;
    extern int optind; 
    extern char *optarg;
    int arg_all = 0;
    int check = 0;

    database = getenv ("IPQUOTADB");
    if (database == NULL) { database = defaultdb; }

    use_syslog = 0;
    pname = argv[0];

    while ((c = getopt(argc,argv,"vhacH")) > 0) 
  	{
	    switch (c)
  		{
		case 'a':
		    arg_all++;
		    break;
		case 'v':
		    version();
		    break; /* never reached */
		case 'c':
		    check++;
		    arg_all++;
		    break;
		case 'H':
		    human = 1;
		    break;
		case 'h':
		default:
		    usage();
		    break; /* never reached */
  		}
  	}

    if (!arg_all) {
	c = argc - optind; 
	if (c) {
	    if (c == 1) {
		arg_all = arg_to_uid (argv[optind], &pwd);
	    } else {
		usage();
	    }
	} else {
	    pwd = getpwuid(getuid());
	}
    }

    if (getuid() && ((arg_all || getuid()!=pwd->pw_uid))) {
	mylog("only root can do that");
	exit (EXIT_FAILURE);
    }

    signal(SIGINT, catch_sig);
    signal(SIGTERM, SIG_IGN); /* ignore this so that we can
				 finish our last accounting run */
    signal(SIGQUIT, catch_sig);
    signal(SIGHUP, catch_sig);

    init_hash();
    init_database(database);	

    do_report (arg_all, check);
	
    close_database(database);
    closelog();
	
    return EXIT_SUCCESS;
}

